# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["MessagingHostedNumberUpdateParams"]


class MessagingHostedNumberUpdateParams(TypedDict, total=False):
    messaging_product: str
    """Configure the messaging product for this number:

    - Omit this field or set its value to `null` to keep the current value.
    - Set this field to a quoted product ID to set this phone number to that product
    """

    messaging_profile_id: str
    """Configure the messaging profile this phone number is assigned to:

    - Omit this field or set its value to `null` to keep the current value.
    - Set this field to `""` to unassign the number from its messaging profile
    - Set this field to a quoted UUID of a messaging profile to assign this number
      to that messaging profile
    """

    tags: SequenceNotStr[str]
    """Tags to set on this phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["FqdnUpdateParams"]


class FqdnUpdateParams(TypedDict, total=False):
    connection_id: str
    """ID of the FQDN connection to which this IP should be attached."""

    dns_record_type: str
    """The DNS record type for the FQDN.

    For cases where a port is not set, the DNS record type must be 'srv'. For cases
    where a port is set, the DNS record type must be 'a'. If the DNS record type is
    'a' and a port is not specified, 5060 will be used.
    """

    fqdn: str
    """FQDN represented by this resource."""

    port: Optional[int]
    """Port to use when connecting to this FQDN."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CampaignSubmitAppealParams"]


class CampaignSubmitAppealParams(TypedDict, total=False):
    appeal_reason: Required[str]
    """
    Detailed explanation of why the campaign should be reconsidered and what changes
    have been made to address the rejection reason.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RcsCapabilities"]


class RcsCapabilities(BaseModel):
    agent_id: Optional[str] = None
    """RCS agent ID"""

    agent_name: Optional[str] = None
    """RCS agent name"""

    features: Optional[List[str]] = None
    """List of RCS capabilities"""

    phone_number: Optional[str] = None
    """Phone number"""

    record_type: Optional[Literal["rcs.capabilities"]] = None
    """Identifies the type of the resource"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["EntityType"]

EntityType: TypeAlias = Literal["PRIVATE_PROFIT", "PUBLIC_PROFIT", "NON_PROFIT", "GOVERNMENT", "SOLE_PROPRIETOR"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["InterfaceStatus"]

InterfaceStatus: TypeAlias = Literal["created", "provisioning", "provisioned", "deleting"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .rcs_to_item import RcsToItem
from .whatsapp_message_content import WhatsappMessageContent

__all__ = ["MessageSendWhatsappResponse", "Data", "DataFrom"]


class DataFrom(BaseModel):
    carrier: Optional[str] = None
    """The carrier of the sender."""

    line_type: Optional[Literal["Wireline", "Wireless", "VoWiFi", "VoIP", "Pre-Paid Wireless", ""]] = None
    """The line-type of the sender."""

    phone_number: Optional[str] = None
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).
    """

    status: Optional[Literal["received", "delivered"]] = None


class Data(BaseModel):
    id: Optional[str] = None
    """message ID"""

    body: Optional[WhatsappMessageContent] = None

    direction: Optional[str] = None

    encoding: Optional[str] = None

    from_: Optional[DataFrom] = FieldInfo(alias="from", default=None)

    messaging_profile_id: Optional[str] = None

    organization_id: Optional[str] = None

    received_at: Optional[datetime] = None

    record_type: Optional[str] = None

    to: Optional[List[RcsToItem]] = None

    type: Optional[str] = None


class MessageSendWhatsappResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SessionRetrieveParams"]


class SessionRetrieveParams(TypedDict, total=False):
    include_participants: bool
    """To decide if room participants should be included in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options

__all__ = ["KnowledgeBasesResource", "AsyncKnowledgeBasesResource"]


class KnowledgeBasesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return KnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return KnowledgeBasesResourceWithStreamingResponse(self)

    def create_knowledge_base(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create a new knowledge base for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a knowledge base from a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a specific knowledge base by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list_knowledge_bases(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List all knowledge bases for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Update a knowledge base definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._put(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncKnowledgeBasesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncKnowledgeBasesResourceWithStreamingResponse(self)

    async def create_knowledge_base(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create a new knowledge base for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a knowledge base from a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a specific knowledge base by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list_knowledge_bases(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List all knowledge bases for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update_knowledge_base(
        self,
        knowledge_base_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Update a knowledge base definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._put(
            f"/ai/missions/{mission_id}/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class KnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create_knowledge_base = to_raw_response_wrapper(
            knowledge_bases.create_knowledge_base,
        )
        self.delete_knowledge_base = to_raw_response_wrapper(
            knowledge_bases.delete_knowledge_base,
        )
        self.get_knowledge_base = to_raw_response_wrapper(
            knowledge_bases.get_knowledge_base,
        )
        self.list_knowledge_bases = to_raw_response_wrapper(
            knowledge_bases.list_knowledge_bases,
        )
        self.update_knowledge_base = to_raw_response_wrapper(
            knowledge_bases.update_knowledge_base,
        )


class AsyncKnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create_knowledge_base = async_to_raw_response_wrapper(
            knowledge_bases.create_knowledge_base,
        )
        self.delete_knowledge_base = async_to_raw_response_wrapper(
            knowledge_bases.delete_knowledge_base,
        )
        self.get_knowledge_base = async_to_raw_response_wrapper(
            knowledge_bases.get_knowledge_base,
        )
        self.list_knowledge_bases = async_to_raw_response_wrapper(
            knowledge_bases.list_knowledge_bases,
        )
        self.update_knowledge_base = async_to_raw_response_wrapper(
            knowledge_bases.update_knowledge_base,
        )


class KnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create_knowledge_base = to_streamed_response_wrapper(
            knowledge_bases.create_knowledge_base,
        )
        self.delete_knowledge_base = to_streamed_response_wrapper(
            knowledge_bases.delete_knowledge_base,
        )
        self.get_knowledge_base = to_streamed_response_wrapper(
            knowledge_bases.get_knowledge_base,
        )
        self.list_knowledge_bases = to_streamed_response_wrapper(
            knowledge_bases.list_knowledge_bases,
        )
        self.update_knowledge_base = to_streamed_response_wrapper(
            knowledge_bases.update_knowledge_base,
        )


class AsyncKnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create_knowledge_base = async_to_streamed_response_wrapper(
            knowledge_bases.create_knowledge_base,
        )
        self.delete_knowledge_base = async_to_streamed_response_wrapper(
            knowledge_bases.delete_knowledge_base,
        )
        self.get_knowledge_base = async_to_streamed_response_wrapper(
            knowledge_bases.get_knowledge_base,
        )
        self.list_knowledge_bases = async_to_streamed_response_wrapper(
            knowledge_bases.list_knowledge_bases,
        )
        self.update_knowledge_base = async_to_streamed_response_wrapper(
            knowledge_bases.update_knowledge_base,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.porting_orders import activation_job_list_params, activation_job_update_params
from ...types.porting_orders_activation_job import PortingOrdersActivationJob
from ...types.porting_orders.activation_job_update_response import ActivationJobUpdateResponse
from ...types.porting_orders.activation_job_retrieve_response import ActivationJobRetrieveResponse

__all__ = ["ActivationJobsResource", "AsyncActivationJobsResource"]


class ActivationJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActivationJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActivationJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActivationJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActivationJobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        activation_job_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActivationJobRetrieveResponse:
        """
        Returns a porting activation job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not activation_job_id:
            raise ValueError(f"Expected a non-empty value for `activation_job_id` but received {activation_job_id!r}")
        return self._get(
            f"/porting_orders/{id}/activation_jobs/{activation_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActivationJobRetrieveResponse,
        )

    def update(
        self,
        activation_job_id: str,
        *,
        id: str,
        activate_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActivationJobUpdateResponse:
        """
        Updates the activation time of a porting activation job.

        Args:
          activate_at: The desired activation time. The activation time should be between any of the
              activation windows.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not activation_job_id:
            raise ValueError(f"Expected a non-empty value for `activation_job_id` but received {activation_job_id!r}")
        return self._patch(
            f"/porting_orders/{id}/activation_jobs/{activation_job_id}",
            body=maybe_transform({"activate_at": activate_at}, activation_job_update_params.ActivationJobUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActivationJobUpdateResponse,
        )

    def list(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortingOrdersActivationJob]:
        """
        Returns a list of your porting activation jobs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/activation_jobs",
            page=SyncDefaultFlatPagination[PortingOrdersActivationJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    activation_job_list_params.ActivationJobListParams,
                ),
            ),
            model=PortingOrdersActivationJob,
        )


class AsyncActivationJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActivationJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActivationJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActivationJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActivationJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        activation_job_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActivationJobRetrieveResponse:
        """
        Returns a porting activation job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not activation_job_id:
            raise ValueError(f"Expected a non-empty value for `activation_job_id` but received {activation_job_id!r}")
        return await self._get(
            f"/porting_orders/{id}/activation_jobs/{activation_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActivationJobRetrieveResponse,
        )

    async def update(
        self,
        activation_job_id: str,
        *,
        id: str,
        activate_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActivationJobUpdateResponse:
        """
        Updates the activation time of a porting activation job.

        Args:
          activate_at: The desired activation time. The activation time should be between any of the
              activation windows.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not activation_job_id:
            raise ValueError(f"Expected a non-empty value for `activation_job_id` but received {activation_job_id!r}")
        return await self._patch(
            f"/porting_orders/{id}/activation_jobs/{activation_job_id}",
            body=await async_maybe_transform(
                {"activate_at": activate_at}, activation_job_update_params.ActivationJobUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActivationJobUpdateResponse,
        )

    def list(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortingOrdersActivationJob, AsyncDefaultFlatPagination[PortingOrdersActivationJob]]:
        """
        Returns a list of your porting activation jobs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/activation_jobs",
            page=AsyncDefaultFlatPagination[PortingOrdersActivationJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    activation_job_list_params.ActivationJobListParams,
                ),
            ),
            model=PortingOrdersActivationJob,
        )


class ActivationJobsResourceWithRawResponse:
    def __init__(self, activation_jobs: ActivationJobsResource) -> None:
        self._activation_jobs = activation_jobs

        self.retrieve = to_raw_response_wrapper(
            activation_jobs.retrieve,
        )
        self.update = to_raw_response_wrapper(
            activation_jobs.update,
        )
        self.list = to_raw_response_wrapper(
            activation_jobs.list,
        )


class AsyncActivationJobsResourceWithRawResponse:
    def __init__(self, activation_jobs: AsyncActivationJobsResource) -> None:
        self._activation_jobs = activation_jobs

        self.retrieve = async_to_raw_response_wrapper(
            activation_jobs.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            activation_jobs.update,
        )
        self.list = async_to_raw_response_wrapper(
            activation_jobs.list,
        )


class ActivationJobsResourceWithStreamingResponse:
    def __init__(self, activation_jobs: ActivationJobsResource) -> None:
        self._activation_jobs = activation_jobs

        self.retrieve = to_streamed_response_wrapper(
            activation_jobs.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            activation_jobs.update,
        )
        self.list = to_streamed_response_wrapper(
            activation_jobs.list,
        )


class AsyncActivationJobsResourceWithStreamingResponse:
    def __init__(self, activation_jobs: AsyncActivationJobsResource) -> None:
        self._activation_jobs = activation_jobs

        self.retrieve = async_to_streamed_response_wrapper(
            activation_jobs.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            activation_jobs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            activation_jobs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .loopcount_param import LoopcountParam

__all__ = ["ActionStartPlaybackParams"]


class ActionStartPlaybackParams(TypedDict, total=False):
    audio_type: Literal["mp3", "wav"]
    """Specifies the type of audio provided in `audio_url` or `playback_content`."""

    audio_url: str
    """The URL of a file to be played back on the call.

    The URL can point to either a WAV or MP3 file. media_name and audio_url cannot
    be used together in one request.
    """

    cache_audio: bool
    """Caches the audio file.

    Useful when playing the same audio file multiple times during the call.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    loop: LoopcountParam
    """The number of times the audio file should be played.

    If supplied, the value must be an integer between 1 and 100, or the special
    string `infinity` for an endless loop.
    """

    media_name: str
    """The media_name of a file to be played back on the call.

    The media_name must point to a file previously uploaded to
    api.telnyx.com/v2/media by the same user/organization. The file must either be a
    WAV or MP3 file.
    """

    overlay: bool
    """
    When enabled, audio will be mixed on top of any other audio that is actively
    being played back. Note that `overlay: true` will only work if there is another
    audio file already being played on the call.
    """

    playback_content: str
    """Allows a user to provide base64 encoded mp3 or wav.

    Note: when using this parameter, `media_url` and `media_name` in the
    `playback_started` and `playback_ended` webhooks will be empty
    """

    stop: str
    """When specified, it stops the current audio being played.

    Specify `current` to stop the current audio being played, and to play the next
    file in the queue. Specify `all` to stop the current audio file being played and
    to also clear all audio files from the queue.
    """

    target_legs: str
    """Specifies the leg or legs on which audio will be played.

    If supplied, the value must be either `self`, `opposite` or `both`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["QueueCreateParams"]


class QueueCreateParams(TypedDict, total=False):
    queue_name: Required[str]
    """The name of the queue. Must be between 1 and 255 characters."""

    max_size: int
    """The maximum number of calls allowed in the queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .reporting.reporting import (
    ReportingResource,
    AsyncReportingResource,
    ReportingResourceWithRawResponse,
    AsyncReportingResourceWithRawResponse,
    ReportingResourceWithStreamingResponse,
    AsyncReportingResourceWithStreamingResponse,
)

__all__ = ["LegacyResource", "AsyncLegacyResource"]


class LegacyResource(SyncAPIResource):
    @cached_property
    def reporting(self) -> ReportingResource:
        return ReportingResource(self._client)

    @cached_property
    def with_raw_response(self) -> LegacyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return LegacyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LegacyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return LegacyResourceWithStreamingResponse(self)


class AsyncLegacyResource(AsyncAPIResource):
    @cached_property
    def reporting(self) -> AsyncReportingResource:
        return AsyncReportingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLegacyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLegacyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLegacyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncLegacyResourceWithStreamingResponse(self)


class LegacyResourceWithRawResponse:
    def __init__(self, legacy: LegacyResource) -> None:
        self._legacy = legacy

    @cached_property
    def reporting(self) -> ReportingResourceWithRawResponse:
        return ReportingResourceWithRawResponse(self._legacy.reporting)


class AsyncLegacyResourceWithRawResponse:
    def __init__(self, legacy: AsyncLegacyResource) -> None:
        self._legacy = legacy

    @cached_property
    def reporting(self) -> AsyncReportingResourceWithRawResponse:
        return AsyncReportingResourceWithRawResponse(self._legacy.reporting)


class LegacyResourceWithStreamingResponse:
    def __init__(self, legacy: LegacyResource) -> None:
        self._legacy = legacy

    @cached_property
    def reporting(self) -> ReportingResourceWithStreamingResponse:
        return ReportingResourceWithStreamingResponse(self._legacy.reporting)


class AsyncLegacyResourceWithStreamingResponse:
    def __init__(self, legacy: AsyncLegacyResource) -> None:
        self._legacy = legacy

    @cached_property
    def reporting(self) -> AsyncReportingResourceWithStreamingResponse:
        return AsyncReportingResourceWithStreamingResponse(self._legacy.reporting)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .porting_event_split_event import PortingEventSplitEvent
from .porting_event_deleted_payload import PortingEventDeletedPayload
from .porting_event_without_webhook import PortingEventWithoutWebhook
from .porting_event_new_comment_event import PortingEventNewCommentEvent
from .porting_event_status_changed_event import PortingEventStatusChangedEvent
from .porting_event_messaging_changed_payload import PortingEventMessagingChangedPayload

__all__ = ["EventListResponse"]

EventListResponse: TypeAlias = Union[
    PortingEventDeletedPayload,
    PortingEventMessagingChangedPayload,
    PortingEventStatusChangedEvent,
    PortingEventNewCommentEvent,
    PortingEventSplitEvent,
    PortingEventWithoutWebhook,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .mission_data import MissionData

__all__ = ["MissionUpdateMissionResponse"]


class MissionUpdateMissionResponse(BaseModel):
    data: MissionData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ....._models import BaseModel
from .plan_step_data import PlanStepData

__all__ = ["PlanAddStepsToPlanResponse"]


class PlanAddStepsToPlanResponse(BaseModel):
    data: List[PlanStepData]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.porting_orders import (
    PortingPhoneNumberBlock,
    PhoneNumberBlockCreateResponse,
    PhoneNumberBlockDeleteResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumberBlocks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        phone_number_block = client.porting_orders.phone_number_blocks.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        )
        assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_blocks.with_raw_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = response.parse()
        assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_blocks.with_streaming_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = response.parse()
            assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_blocks.with_raw_response.create(
                porting_order_id="",
                activation_ranges=[
                    {
                        "end_at": "+4930244999910",
                        "start_at": "+4930244999901",
                    }
                ],
                phone_number_range={
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number_block = client.porting_orders.phone_number_blocks.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number_block = client.porting_orders.phone_number_blocks.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={
                "activation_status": "Active",
                "phone_number": ["+12003151212"],
                "portability_status": "confirmed",
                "porting_order_id": ["f3575e15-32ce-400e-a4c0-dd78800c20b0"],
                "status": "in-process",
                "support_key": "sr_a12345",
            },
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(SyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_blocks.with_raw_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_blocks.with_streaming_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_blocks.with_raw_response.list(
                porting_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        phone_number_block = client.porting_orders.phone_number_blocks.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_blocks.with_raw_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = response.parse()
        assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_blocks.with_streaming_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = response.parse()
            assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_blocks.with_raw_response.delete(
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                porting_order_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.phone_number_blocks.with_raw_response.delete(
                id="",
                porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncPhoneNumberBlocks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        phone_number_block = await async_client.porting_orders.phone_number_blocks.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        )
        assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_blocks.with_raw_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = await response.parse()
        assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_blocks.with_streaming_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                }
            ],
            phone_number_range={
                "end_at": "+4930244999910",
                "start_at": "+4930244999901",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = await response.parse()
            assert_matches_type(PhoneNumberBlockCreateResponse, phone_number_block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_blocks.with_raw_response.create(
                porting_order_id="",
                activation_ranges=[
                    {
                        "end_at": "+4930244999910",
                        "start_at": "+4930244999901",
                    }
                ],
                phone_number_range={
                    "end_at": "+4930244999910",
                    "start_at": "+4930244999901",
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number_block = await async_client.porting_orders.phone_number_blocks.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AsyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_block = await async_client.porting_orders.phone_number_blocks.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={
                "activation_status": "Active",
                "phone_number": ["+12003151212"],
                "portability_status": "confirmed",
                "porting_order_id": ["f3575e15-32ce-400e-a4c0-dd78800c20b0"],
                "status": "in-process",
                "support_key": "sr_a12345",
            },
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(AsyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_blocks.with_raw_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_blocks.with_streaming_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[PortingPhoneNumberBlock], phone_number_block, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_blocks.with_raw_response.list(
                porting_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        phone_number_block = await async_client.porting_orders.phone_number_blocks.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_blocks.with_raw_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_block = await response.parse()
        assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_blocks.with_streaming_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_block = await response.parse()
            assert_matches_type(PhoneNumberBlockDeleteResponse, phone_number_block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_blocks.with_raw_response.delete(
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                porting_order_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.phone_number_blocks.with_raw_response.delete(
                id="",
                porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionStopGatherParams"]


class ActionStopGatherParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .customer_service_record import CustomerServiceRecord

__all__ = ["CustomerServiceRecordRetrieveResponse"]


class CustomerServiceRecordRetrieveResponse(BaseModel):
    data: Optional[CustomerServiceRecord] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["ReportListWdrsParams"]


class ReportListWdrsParams(TypedDict, total=False):
    id: str
    """WDR uuid"""

    end_date: str
    """End date"""

    imsi: str
    """International mobile subscriber identity"""

    mcc: str
    """Mobile country code"""

    mnc: str
    """Mobile network code"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    phone_number: str
    """Phone number"""

    sim_card_id: str
    """Sim card unique identifier"""

    sim_group_id: str
    """Sim group unique identifier"""

    sim_group_name: str
    """Sim group name"""

    sort: SequenceNotStr[str]
    """Field used to order the data.

    If no field is specified, default value is 'created_at'
    """

    start_date: str
    """Start date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .notification_profile import NotificationProfile

__all__ = ["NotificationProfileCreateResponse"]


class NotificationProfileCreateResponse(BaseModel):
    data: Optional[NotificationProfile] = None
    """A Collection of Notification Channels"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ConnectionNoiseSuppressionDetails"]


class ConnectionNoiseSuppressionDetails(BaseModel):
    """Configuration options for noise suppression.

    These settings are stored regardless of the noise_suppression value, but only take effect when noise_suppression is not 'disabled'. If you disable noise suppression and later re-enable it, the previously configured settings will be used.
    """

    attenuation_limit: Optional[int] = None
    """The attenuation limit value for the selected engine.

    Default values vary by engine: 0 for 'denoiser', 80 for 'deep_filter_net',
    'deep_filter_net_large', and all Krisp engines ('krisp_viva_tel',
    'krisp_viva_tel_lite', 'krisp_viva_promodel', 'krisp_viva_ss'), 100 for
    'quail_voice_focus'.
    """

    engine: Optional[
        Literal[
            "denoiser",
            "deep_filter_net",
            "deep_filter_net_large",
            "krisp_viva_tel",
            "krisp_viva_tel_lite",
            "krisp_viva_promodel",
            "krisp_viva_ss",
            "quail_voice_focus",
        ]
    ] = None
    """The noise suppression engine to use.

    'denoiser' is the default engine. 'deep_filter_net' and 'deep_filter_net_large'
    are alternative engines with different performance characteristics. Krisp
    engines ('krisp_viva_tel', 'krisp_viva_tel_lite', 'krisp_viva_promodel',
    'krisp_viva_ss') provide advanced noise suppression capabilities.
    'quail_voice_focus' provides Quail-based voice focus noise suppression.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ExternalConnectionPhoneNumber"]


class ExternalConnectionPhoneNumber(BaseModel):
    acquired_capabilities: Optional[
        List[Literal["FirstPartyAppAssignment", "InboundCalling", "Office365", "OutboundCalling", "UserAssignment"]]
    ] = None

    civic_address_id: Optional[str] = None
    """Identifies the civic address assigned to the phone number."""

    displayed_country_code: Optional[str] = None
    """
    The iso country code that will be displayed to the user when they receive a call
    from this phone number.
    """

    location_id: Optional[str] = None
    """Identifies the location assigned to the phone number."""

    number_id: Optional[str] = None
    """Phone number ID from the Telnyx API."""

    telephone_number: Optional[str] = None
    """Phone number in E164 format."""

    ticket_id: Optional[str] = None
    """Uniquely identifies the resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AutoRechargePref"]


class AutoRechargePref(BaseModel):
    id: Optional[str] = None
    """The unique identifier for the auto recharge preference."""

    enabled: Optional[bool] = None
    """Whether auto recharge is enabled."""

    invoice_enabled: Optional[bool] = None

    preference: Optional[Literal["credit_paypal", "ach"]] = None
    """The payment preference for auto recharge."""

    recharge_amount: Optional[str] = None
    """
    The amount to recharge the account, the actual recharge amount will be the
    amount necessary to reach the threshold amount plus the recharge amount.
    """

    record_type: Optional[str] = None
    """The record type."""

    threshold_amount: Optional[str] = None
    """The threshold amount at which the account will be recharged."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultPaginationForLogMessages, AsyncDefaultPaginationForLogMessages
from ..._base_client import AsyncPaginator, make_request_options
from ...types.external_connections import log_message_list_params
from ...types.external_connections.log_message_list_response import LogMessageListResponse
from ...types.external_connections.log_message_dismiss_response import LogMessageDismissResponse
from ...types.external_connections.log_message_retrieve_response import LogMessageRetrieveResponse

__all__ = ["LogMessagesResource", "AsyncLogMessagesResource"]


class LogMessagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LogMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return LogMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return LogMessagesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogMessageRetrieveResponse:
        """
        Retrieve a log message for an external connection associated with your account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/external_connections/log_messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogMessageRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: log_message_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPaginationForLogMessages[LogMessageListResponse]:
        """
        Retrieve a list of log messages for all external connections associated with
        your account.

        Args:
          filter: Filter parameter for log messages (deepObject style). Supports filtering by
              external_connection_id and telephone_number with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/external_connections/log_messages",
            page=SyncDefaultPaginationForLogMessages[LogMessageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    log_message_list_params.LogMessageListParams,
                ),
            ),
            model=LogMessageListResponse,
        )

    def dismiss(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogMessageDismissResponse:
        """
        Dismiss a log message for an external connection associated with your account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/external_connections/log_messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogMessageDismissResponse,
        )


class AsyncLogMessagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLogMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncLogMessagesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogMessageRetrieveResponse:
        """
        Retrieve a log message for an external connection associated with your account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/external_connections/log_messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogMessageRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: log_message_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LogMessageListResponse, AsyncDefaultPaginationForLogMessages[LogMessageListResponse]]:
        """
        Retrieve a list of log messages for all external connections associated with
        your account.

        Args:
          filter: Filter parameter for log messages (deepObject style). Supports filtering by
              external_connection_id and telephone_number with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/external_connections/log_messages",
            page=AsyncDefaultPaginationForLogMessages[LogMessageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    log_message_list_params.LogMessageListParams,
                ),
            ),
            model=LogMessageListResponse,
        )

    async def dismiss(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogMessageDismissResponse:
        """
        Dismiss a log message for an external connection associated with your account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/external_connections/log_messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogMessageDismissResponse,
        )


class LogMessagesResourceWithRawResponse:
    def __init__(self, log_messages: LogMessagesResource) -> None:
        self._log_messages = log_messages

        self.retrieve = to_raw_response_wrapper(
            log_messages.retrieve,
        )
        self.list = to_raw_response_wrapper(
            log_messages.list,
        )
        self.dismiss = to_raw_response_wrapper(
            log_messages.dismiss,
        )


class AsyncLogMessagesResourceWithRawResponse:
    def __init__(self, log_messages: AsyncLogMessagesResource) -> None:
        self._log_messages = log_messages

        self.retrieve = async_to_raw_response_wrapper(
            log_messages.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            log_messages.list,
        )
        self.dismiss = async_to_raw_response_wrapper(
            log_messages.dismiss,
        )


class LogMessagesResourceWithStreamingResponse:
    def __init__(self, log_messages: LogMessagesResource) -> None:
        self._log_messages = log_messages

        self.retrieve = to_streamed_response_wrapper(
            log_messages.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            log_messages.list,
        )
        self.dismiss = to_streamed_response_wrapper(
            log_messages.dismiss,
        )


class AsyncLogMessagesResourceWithStreamingResponse:
    def __init__(self, log_messages: AsyncLogMessagesResource) -> None:
        self._log_messages = log_messages

        self.retrieve = async_to_streamed_response_wrapper(
            log_messages.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            log_messages.list,
        )
        self.dismiss = async_to_streamed_response_wrapper(
            log_messages.dismiss,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .number_lookup import (
    NumberLookupResource,
    AsyncNumberLookupResource,
    NumberLookupResourceWithRawResponse,
    AsyncNumberLookupResourceWithRawResponse,
    NumberLookupResourceWithStreamingResponse,
    AsyncNumberLookupResourceWithStreamingResponse,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting import usage_report_retrieve_speech_to_text_params
from .....types.legacy.reporting.usage_report_retrieve_speech_to_text_response import (
    UsageReportRetrieveSpeechToTextResponse,
)

__all__ = ["UsageReportsResource", "AsyncUsageReportsResource"]


class UsageReportsResource(SyncAPIResource):
    @cached_property
    def messaging(self) -> MessagingResource:
        return MessagingResource(self._client)

    @cached_property
    def number_lookup(self) -> NumberLookupResource:
        return NumberLookupResource(self._client)

    @cached_property
    def voice(self) -> VoiceResource:
        return VoiceResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UsageReportsResourceWithStreamingResponse(self)

    def retrieve_speech_to_text(
        self,
        *,
        end_date: Union[str, datetime] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportRetrieveSpeechToTextResponse:
        """Generate and fetch speech to text usage report synchronously.

        This endpoint will
        both generate and fetch the speech to text report over a specified time period.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/legacy/reporting/usage_reports/speech_to_text",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    usage_report_retrieve_speech_to_text_params.UsageReportRetrieveSpeechToTextParams,
                ),
            ),
            cast_to=UsageReportRetrieveSpeechToTextResponse,
        )


class AsyncUsageReportsResource(AsyncAPIResource):
    @cached_property
    def messaging(self) -> AsyncMessagingResource:
        return AsyncMessagingResource(self._client)

    @cached_property
    def number_lookup(self) -> AsyncNumberLookupResource:
        return AsyncNumberLookupResource(self._client)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        return AsyncVoiceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUsageReportsResourceWithStreamingResponse(self)

    async def retrieve_speech_to_text(
        self,
        *,
        end_date: Union[str, datetime] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportRetrieveSpeechToTextResponse:
        """Generate and fetch speech to text usage report synchronously.

        This endpoint will
        both generate and fetch the speech to text report over a specified time period.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/legacy/reporting/usage_reports/speech_to_text",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    usage_report_retrieve_speech_to_text_params.UsageReportRetrieveSpeechToTextParams,
                ),
            ),
            cast_to=UsageReportRetrieveSpeechToTextResponse,
        )


class UsageReportsResourceWithRawResponse:
    def __init__(self, usage_reports: UsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.retrieve_speech_to_text = to_raw_response_wrapper(
            usage_reports.retrieve_speech_to_text,
        )

    @cached_property
    def messaging(self) -> MessagingResourceWithRawResponse:
        return MessagingResourceWithRawResponse(self._usage_reports.messaging)

    @cached_property
    def number_lookup(self) -> NumberLookupResourceWithRawResponse:
        return NumberLookupResourceWithRawResponse(self._usage_reports.number_lookup)

    @cached_property
    def voice(self) -> VoiceResourceWithRawResponse:
        return VoiceResourceWithRawResponse(self._usage_reports.voice)


class AsyncUsageReportsResourceWithRawResponse:
    def __init__(self, usage_reports: AsyncUsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.retrieve_speech_to_text = async_to_raw_response_wrapper(
            usage_reports.retrieve_speech_to_text,
        )

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithRawResponse:
        return AsyncMessagingResourceWithRawResponse(self._usage_reports.messaging)

    @cached_property
    def number_lookup(self) -> AsyncNumberLookupResourceWithRawResponse:
        return AsyncNumberLookupResourceWithRawResponse(self._usage_reports.number_lookup)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithRawResponse:
        return AsyncVoiceResourceWithRawResponse(self._usage_reports.voice)


class UsageReportsResourceWithStreamingResponse:
    def __init__(self, usage_reports: UsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.retrieve_speech_to_text = to_streamed_response_wrapper(
            usage_reports.retrieve_speech_to_text,
        )

    @cached_property
    def messaging(self) -> MessagingResourceWithStreamingResponse:
        return MessagingResourceWithStreamingResponse(self._usage_reports.messaging)

    @cached_property
    def number_lookup(self) -> NumberLookupResourceWithStreamingResponse:
        return NumberLookupResourceWithStreamingResponse(self._usage_reports.number_lookup)

    @cached_property
    def voice(self) -> VoiceResourceWithStreamingResponse:
        return VoiceResourceWithStreamingResponse(self._usage_reports.voice)


class AsyncUsageReportsResourceWithStreamingResponse:
    def __init__(self, usage_reports: AsyncUsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.retrieve_speech_to_text = async_to_streamed_response_wrapper(
            usage_reports.retrieve_speech_to_text,
        )

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithStreamingResponse:
        return AsyncMessagingResourceWithStreamingResponse(self._usage_reports.messaging)

    @cached_property
    def number_lookup(self) -> AsyncNumberLookupResourceWithStreamingResponse:
        return AsyncNumberLookupResourceWithStreamingResponse(self._usage_reports.number_lookup)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithStreamingResponse:
        return AsyncVoiceResourceWithStreamingResponse(self._usage_reports.voice)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["InexplicitNumberOrderCreateParams", "OrderingGroup", "OrderingGroupPhoneNumber"]


class InexplicitNumberOrderCreateParams(TypedDict, total=False):
    ordering_groups: Required[Iterable[OrderingGroup]]
    """Group(s) of numbers to order.

    You can have multiple ordering_groups objects added to a single request.
    """

    billing_group_id: str
    """Billing group id to apply to phone numbers that are purchased"""

    connection_id: str
    """Connection id to apply to phone numbers that are purchased"""

    customer_reference: str
    """Reference label for the customer"""

    messaging_profile_id: str
    """Messaging profile id to apply to phone numbers that are purchased"""


class OrderingGroupPhoneNumber(TypedDict, total=False):
    """Phone number search criteria"""

    contains: str
    """Filter for phone numbers that contain the digits specified"""

    ends_with: str
    """Filter by the ending digits of the phone number"""

    starts_with: str
    """Filter by the starting digits of the phone number"""


class OrderingGroup(TypedDict, total=False):
    count_requested: Required[str]
    """Quantity of phone numbers to order"""

    country_iso: Required[Literal["US", "CA"]]
    """Country where you would like to purchase phone numbers.

    Allowable values: US, CA
    """

    phone_number_type: Required[str]
    """Number type (local, toll-free, etc.)"""

    administrative_area: str
    """Filter for phone numbers in a given state / province"""

    exclude_held_numbers: bool
    """
    Filter to exclude phone numbers that are currently on hold/reserved for your
    account.
    """

    features: SequenceNotStr[str]
    """
    Filter for phone numbers that have the features to satisfy your use case (e.g.,
    ["voice"])
    """

    locality: str
    """Filter for phone numbers in a given city / region / rate center"""

    national_destination_code: str
    """Filter by area code"""

    phone_number: OrderingGroupPhoneNumber
    """Phone number search criteria"""

    quickship: bool
    """
    Filter to exclude phone numbers that need additional time after to purchase to
    activate. Only applicable for +1 toll_free numbers.
    """

    strategy: Literal["always", "never"]
    """Ordering strategy.

    Define what action should be taken if we don't have enough phone numbers to
    fulfill your request. Allowable values are: always = proceed with ordering phone
    numbers, regardless of current inventory levels; never = do not place any orders
    unless there are enough phone numbers to satisfy the request. If not specified,
    the always strategy will be enforced.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .replaced_link_click import ReplacedLinkClick

__all__ = ["ReplacedLinkClickWebhookEvent"]


class ReplacedLinkClickWebhookEvent(BaseModel):
    data: Optional[ReplacedLinkClick] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .plan import (
    PlanResource,
    AsyncPlanResource,
    PlanResourceWithRawResponse,
    AsyncPlanResourceWithRawResponse,
    PlanResourceWithStreamingResponse,
    AsyncPlanResourceWithStreamingResponse,
)
from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .telnyx_agents import (
    TelnyxAgentsResource,
    AsyncTelnyxAgentsResource,
    TelnyxAgentsResourceWithRawResponse,
    AsyncTelnyxAgentsResourceWithRawResponse,
    TelnyxAgentsResourceWithStreamingResponse,
    AsyncTelnyxAgentsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "PlanResource",
    "AsyncPlanResource",
    "PlanResourceWithRawResponse",
    "AsyncPlanResourceWithRawResponse",
    "PlanResourceWithStreamingResponse",
    "AsyncPlanResourceWithStreamingResponse",
    "TelnyxAgentsResource",
    "AsyncTelnyxAgentsResource",
    "TelnyxAgentsResourceWithRawResponse",
    "AsyncTelnyxAgentsResourceWithRawResponse",
    "TelnyxAgentsResourceWithStreamingResponse",
    "AsyncTelnyxAgentsResourceWithStreamingResponse",
    "RunsResource",
    "AsyncRunsResource",
    "RunsResourceWithRawResponse",
    "AsyncRunsResourceWithRawResponse",
    "RunsResourceWithStreamingResponse",
    "AsyncRunsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from .telnyx_agent_data import TelnyxAgentData

__all__ = ["TelnyxAgentLinkResponse"]


class TelnyxAgentLinkResponse(BaseModel):
    data: TelnyxAgentData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Cursor"]


class Cursor(BaseModel):
    after: Optional[str] = None
    """Opaque identifier of next page."""

    before: Optional[str] = None
    """Opaque identifier of previous page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AutorespConfigCreateParams"]


class AutorespConfigCreateParams(TypedDict, total=False):
    country_code: Required[str]

    keywords: Required[SequenceNotStr[str]]

    op: Required[Literal["start", "stop", "info"]]

    resp_text: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.external_connections import (
    Upload,
    UploadRetryResponse,
    UploadCreateResponse,
    UploadRetrieveResponse,
    UploadPendingCountResponse,
    UploadRefreshStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUploads:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        )
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
            additional_usages=["calling_user_assignment"],
            civic_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            location_id="67ea7693-9cd5-4a68-8c76-abb3aa5bf5d2",
            usage="first_party_app_assignment",
        )
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(UploadCreateResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.create(
                id="",
                number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )
        assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.retrieve(
                ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `ticket_id` but received ''"):
            client.external_connections.uploads.with_raw_response.retrieve(
                ticket_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.list(
            id="1293384261075731499",
        )
        assert_matches_type(SyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.list(
            id="1293384261075731499",
            filter={
                "civic_address_id": {"eq": "19990261512338516954"},
                "location_id": {"eq": "19995665508264022121"},
                "phone_number": {
                    "contains": "+1970",
                    "eq": "+19705555098",
                },
                "status": {"eq": ["pending_upload", "pending"]},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.list(
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.list(
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[Upload], upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_pending_count(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.pending_count(
            "1293384261075731499",
        )
        assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_pending_count(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.pending_count(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_pending_count(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.pending_count(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_pending_count(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.pending_count(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_refresh_status(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.refresh_status(
            "1293384261075731499",
        )
        assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_refresh_status(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.refresh_status(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_refresh_status(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.refresh_status(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_refresh_status(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.refresh_status(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retry(self, client: Telnyx) -> None:
        upload = client.external_connections.uploads.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )
        assert_matches_type(UploadRetryResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retry(self, client: Telnyx) -> None:
        response = client.external_connections.uploads.with_raw_response.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = response.parse()
        assert_matches_type(UploadRetryResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retry(self, client: Telnyx) -> None:
        with client.external_connections.uploads.with_streaming_response.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = response.parse()
            assert_matches_type(UploadRetryResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retry(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.uploads.with_raw_response.retry(
                ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `ticket_id` but received ''"):
            client.external_connections.uploads.with_raw_response.retry(
                ticket_id="",
                id="1293384261075731499",
            )


class TestAsyncUploads:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        )
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
            additional_usages=["calling_user_assignment"],
            civic_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            location_id="67ea7693-9cd5-4a68-8c76-abb3aa5bf5d2",
            usage="first_party_app_assignment",
        )
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(UploadCreateResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.create(
            id="1293384261075731499",
            number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(UploadCreateResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.create(
                id="",
                number_ids=["3920457616934164700", "3920457616934164701", "3920457616934164702", "3920457616934164703"],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )
        assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.retrieve(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(UploadRetrieveResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.retrieve(
                ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `ticket_id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.retrieve(
                ticket_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.list(
            id="1293384261075731499",
        )
        assert_matches_type(AsyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.list(
            id="1293384261075731499",
            filter={
                "civic_address_id": {"eq": "19990261512338516954"},
                "location_id": {"eq": "19995665508264022121"},
                "phone_number": {
                    "contains": "+1970",
                    "eq": "+19705555098",
                },
                "status": {"eq": ["pending_upload", "pending"]},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.list(
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[Upload], upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.list(
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[Upload], upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_pending_count(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.pending_count(
            "1293384261075731499",
        )
        assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_pending_count(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.pending_count(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_pending_count(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.pending_count(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(UploadPendingCountResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_pending_count(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.pending_count(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_refresh_status(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.refresh_status(
            "1293384261075731499",
        )
        assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_refresh_status(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.refresh_status(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_refresh_status(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.refresh_status(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(UploadRefreshStatusResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_refresh_status(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.refresh_status(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retry(self, async_client: AsyncTelnyx) -> None:
        upload = await async_client.external_connections.uploads.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )
        assert_matches_type(UploadRetryResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retry(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.uploads.with_raw_response.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload = await response.parse()
        assert_matches_type(UploadRetryResponse, upload, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retry(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.uploads.with_streaming_response.retry(
            ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload = await response.parse()
            assert_matches_type(UploadRetryResponse, upload, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retry(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.retry(
                ticket_id="7b6a6449-b055-45a6-81f6-f6f0dffa4cc6",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `ticket_id` but received ''"):
            await async_client.external_connections.uploads.with_raw_response.retry(
                ticket_id="",
                id="1293384261075731499",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    ManagedAccountListResponse,
    ManagedAccountCreateResponse,
    ManagedAccountUpdateResponse,
    ManagedAccountRetrieveResponse,
    ManagedAccountUpdateGlobalChannelLimitResponse,
    ManagedAccountGetAllocatableGlobalOutboundChannelsResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestManagedAccounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.create(
            business_name="Larry's Cat Food Inc",
        )
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.create(
            business_name="Larry's Cat Food Inc",
            email="larry_cat_food@customer.org",
            managed_account_allow_custom_pricing=False,
            password="3jVjLq!tMuWKyWx4NN*CvhnB",
            rollup_billing=False,
        )
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.create(
            business_name="Larry's Cat Food Inc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.create(
            business_name="Larry's Cat Food Inc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.retrieve(
            "id",
        )
        assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.managed_accounts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.update(
            id="id",
        )
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.update(
            id="id",
            managed_account_allow_custom_pricing=True,
        )
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.managed_accounts.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.list()
        assert_matches_type(SyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.list(
            filter={
                "email": {
                    "contains": "john",
                    "eq": "eq",
                },
                "organization_name": {
                    "contains": "contains",
                    "eq": "Example Company LLC",
                },
            },
            include_cancelled_accounts=True,
            page_number=0,
            page_size=0,
            sort="email",
        )
        assert_matches_type(SyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_allocatable_global_outbound_channels(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.get_allocatable_global_outbound_channels()
        assert_matches_type(
            ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_allocatable_global_outbound_channels(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.get_allocatable_global_outbound_channels()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(
            ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_allocatable_global_outbound_channels(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.get_allocatable_global_outbound_channels() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(
                ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_global_channel_limit(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.update_global_channel_limit(
            id="id",
        )
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_global_channel_limit_with_all_params(self, client: Telnyx) -> None:
        managed_account = client.managed_accounts.update_global_channel_limit(
            id="id",
            channel_limit=30,
        )
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_global_channel_limit(self, client: Telnyx) -> None:
        response = client.managed_accounts.with_raw_response.update_global_channel_limit(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = response.parse()
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_global_channel_limit(self, client: Telnyx) -> None:
        with client.managed_accounts.with_streaming_response.update_global_channel_limit(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = response.parse()
            assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_global_channel_limit(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.managed_accounts.with_raw_response.update_global_channel_limit(
                id="",
            )


class TestAsyncManagedAccounts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.create(
            business_name="Larry's Cat Food Inc",
        )
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.create(
            business_name="Larry's Cat Food Inc",
            email="larry_cat_food@customer.org",
            managed_account_allow_custom_pricing=False,
            password="3jVjLq!tMuWKyWx4NN*CvhnB",
            rollup_billing=False,
        )
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.create(
            business_name="Larry's Cat Food Inc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.managed_accounts.with_streaming_response.create(
            business_name="Larry's Cat Food Inc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(ManagedAccountCreateResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.retrieve(
            "id",
        )
        assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.managed_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(ManagedAccountRetrieveResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.managed_accounts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.update(
            id="id",
        )
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.update(
            id="id",
            managed_account_allow_custom_pricing=True,
        )
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.managed_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(ManagedAccountUpdateResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.managed_accounts.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.list()
        assert_matches_type(AsyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.list(
            filter={
                "email": {
                    "contains": "john",
                    "eq": "eq",
                },
                "organization_name": {
                    "contains": "contains",
                    "eq": "Example Company LLC",
                },
            },
            include_cancelled_accounts=True,
            page_number=0,
            page_size=0,
            sort="email",
        )
        assert_matches_type(AsyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.managed_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[ManagedAccountListResponse], managed_account, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_allocatable_global_outbound_channels(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.get_allocatable_global_outbound_channels()
        assert_matches_type(
            ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_allocatable_global_outbound_channels(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.get_allocatable_global_outbound_channels()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(
            ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_allocatable_global_outbound_channels(self, async_client: AsyncTelnyx) -> None:
        async with (
            async_client.managed_accounts.with_streaming_response.get_allocatable_global_outbound_channels()
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(
                ManagedAccountGetAllocatableGlobalOutboundChannelsResponse, managed_account, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_global_channel_limit(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.update_global_channel_limit(
            id="id",
        )
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_global_channel_limit_with_all_params(self, async_client: AsyncTelnyx) -> None:
        managed_account = await async_client.managed_accounts.update_global_channel_limit(
            id="id",
            channel_limit=30,
        )
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_global_channel_limit(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.managed_accounts.with_raw_response.update_global_channel_limit(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        managed_account = await response.parse()
        assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_global_channel_limit(self, async_client: AsyncTelnyx) -> None:
        async with async_client.managed_accounts.with_streaming_response.update_global_channel_limit(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            managed_account = await response.parse()
            assert_matches_type(ManagedAccountUpdateGlobalChannelLimitResponse, managed_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_global_channel_limit(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.managed_accounts.with_raw_response.update_global_channel_limit(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel
from .shared.room_participant import RoomParticipant

__all__ = ["RoomSession"]


class RoomSession(BaseModel):
    id: Optional[str] = None
    """A unique identifier for the room session."""

    active: Optional[bool] = None
    """Shows if the room session is active or not."""

    created_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room session was created."""

    ended_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room session has ended."""

    participants: Optional[List[RoomParticipant]] = None

    record_type: Optional[str] = None

    room_id: Optional[str] = None
    """Identify the room hosting that room session."""

    updated_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room session was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .verify_meta import VerifyMeta
from ..verification import Verification

__all__ = ["ByPhoneNumberListResponse"]


class ByPhoneNumberListResponse(BaseModel):
    data: List[Verification]

    meta: VerifyMeta
